/***************************************************************************/
/**
	@file		AVR_USART.C
	@brief		Simple implementation of an command-interface
	@brief		Target/Compiler: AVR_MEGA / GCC(WinAVR)
	@author		Gerhard Bruenner
	@version	V3.1

	@date  2009.11.20  - new release - new setup
    @date  2010.06.28  - added defines for AVR M 644

	@details	Routines for the internal hardware UART.

	@bug		none that I am aware so far
\***************************************************************************/

#define dAVR_USART_C

/* Includes
****************************************************************************/
	#include "AVR_USART.h"

/* Global Variables
****************************************************************************/

/* 	Functions
****************************************************************************/

#ifdef dAVR_USART_INCLUDE_INIT_FUNCTION   
	/**
		@brief		basic initialisation function for USART
		@param		none - must be changed to what ever it shall do
		@return		void

		@details    This function must be adopted to what ever setting is required.
					This is for the sake of code-size.
	 */
	void avr_usart_init_9600_8_1_N_Asyn( void )	{

		/* Set UART-Control-Registers 
										\todo this code is a little bit smaller without those macros 
        */
			dAVR_USART_ENABLE_TRANSMIT();
			dAVR_USART_ENABLE_RECEIVE(); 
			dAVR_USART_ENABLE_RECV_IRQ();
			dAVR_USART_DISABLE_TRANS_IRQ();
			dAVR_USART_MODE_ASYNCHRONOUS();
			dAVR_USART_SET_PARITY_NONE();
			dAVR_USART_SET_DATABITS_TO_8();
			dAVR_USART_SET_STOPPBITS_1();
			dAVR_USART_SAMPLE_AT_POS_EDGE();			

		/* Set Baudrate	*/
			UBRRH = (uint8_t)(dAVR_USART_UBRR_CALC(9600)>>8); /*9600UL 19200UL 57600UL 115200UL*/
			UBRRL = (uint8_t) dAVR_USART_UBRR_CALC(9600);
	} /* void init_usart( void ) */
#endif

/***********************************************************************************\
	Send Routines
\***********************************************************************************/

	/**
	 	@brief		sends a single byte
	 	@param		uint8_t data - byte to be send
	 	@return		void
	 	@details    none
	 */
	void avr_usart_putc(uint8_t data){
		while ((UCSRA & 1<<UDRE) == 0); /* warten bis Senden moeglich */
		UDR = data; /* schreibt das Zeichen x auf die Schnittstelle */
	}/*avr_usart_putc*/



	/**
	 	@brief		transmits each character of a string terminated with null
	 	@param		char *s - string to be send
	 	@return		void
	 	@details    none
	 */
	void avr_usart_puts (char *s){
		while (*s){ /* so lange *s != '\0' also ungleich dem "String-Endezeichen" */
			avr_usart_putc((uint8_t) *s);
			s++;
		}
	}/*avr_usart_puts*/



/***********************************************************************************\
	Receive Routines
\***********************************************************************************/

	/**
	 	@brief		wait till one char has been received
	 	@param		void
	 	@return		uint8_t - byte received
	 	@details    none
	 */
	uint8_t avr_usart_getc_wait(void)	{
		while (!(UCSRA & (1<<RXC))); /*  warten bis Zeichen verfuegbar */
		return UDR; /* Zeichen aus UDR an Aufrufer zurueckgeben */
	}/*avr_usart_getc_wait*/



	/**
	 	@brief		checks whether something has been received
	 	@param		void
	 	@return		uint8_t >0 -> byte received
	 	@details    none
	 */
	uint8_t avr_usart_getc(void)	{
		if (UCSRA & (1<<RXC)){ /* Zeichen verfuegbar ? */
				return UDR; /* Zeichen aus UDR an Aufrufer zurueckgeben */
		}else	return 0;
	}/*avr_usart_getc*/



	/**
	 	@brief		interrupt service routine to be used to receive characters in ISR
	 	@param		void
	 	@return		uint8_t >0 -> byte received
	 	@details    none
	 */
	inline uint8_t avr_usart_getc_isr(void){
		return UDR; /* Zeichen aus UDR an Aufrufer zurueckgeben */
	}/*avr_usart_getc_isr*/


/***********************************************************************************\
	IRQ-Routines examples
	void 	(*isr_rcv_routine)	(uint8_t byte);	// function pointer to routine which shall be called in receive-IRQ
	uint8_t (*isr_tx_routine)	(void);			// function pointer to routine which shall be called in transmit-IRQ

	ISR (USART_RX){
		uint8_t ui8_recv=avr_usart_getc_isr();
		if (NULL!=*isr_rcv_routine) isr_rcv_routine(ui8_recv);
	}

	ISR (USART_TX_vect){
		if (NULL!=*isr_rcv_routine) isr_rcv_routine(ui8_recv);
	}


\***********************************************************************************/



#undef dAVR_USART_C


	/* USART Control Register A - Settings
	************************************************************************/

		/* Register A wird nur beim Empfang und Versenden bentigt.*/

	/* USART Control Register B - Settings
		Bit 7 		6 		5 		4 		3 		2 		1 		0
			RXCIE 	TXCIE 	UDRIE 	RXEN 	TXEN 	UCSZ2 	RXB8 	TXB8
		Bit 7 - RXCIE: RX Complete Interrupt Enable
		Bit 6 - TXCIE: TX Complete Interrupt Enable
		Bit 5 - UDRIE: USART Data Register Empty Interrupt Enable
		Bit 4 - RXEN: Receiver Enable
		Bit 3 - TXEN: Transmitter Enable
		Bit 2 - UCSZ2: Character Size wird bei Verwendung von 9 Datenbits gesetzt
		Bit 1 - RXB8: Receive Data Bit - contains 9th databit
		Bit 0 - TXB8: Transmit Data Bit - contains 9th databit
	************************************************************************/
	/* USART Control Register C - Settings
		Bit 7 		6 		5 		4 		3 		2 		1 		0
			URSEL 	UMSEL 	UPM1 	UPM0 	USBS 	UCSZ1 	UCSZ0 	UCPOL
		The UCSRC Register shares the same I/O location as the UBRRH Register.
		Bit 7 URSEL: Register Select
			This bit selects between accessing the UCSRC or the UBRRH Register.
			It is read as one when reading UCSRC. The URSEL must be one when writing the UCSRC.
		Bit 6  UMSEL: USART Mode Select
		This bit selects between Asynchronous and Synchronous mode of operation.
	************************************************************************/
