/***************************************************************************/
/**
	@file		AVR_USART.H
	@brief		Simple implementation of an command-interface
	@brief		Target/Compiler: AVR_MEGA / GCC(WinAVR)
	@author		Gerhard Bruenner
	@version	V3.1

	@date  2009.11.20  - new release - new setup
    @date  2010.06.28  - added defines for AVR M 644

	@details	Routines for the internal hardware UART.

	@bug		none that I am aware so far

\***************************************************************************/

#ifndef dAVR_USART_H
	#define dAVR_USART_H	/**< precompiler help */


/* Defines for Code-Control
 ****************************************************************************/
	#define dAVR_USART_INCLUDE_INIT_FUNCTION  1 /* excludes or includs the function avr_usart_init_9600_8_1_N_Asyn */

/* Include-Dateien
****************************************************************************/
	#include <avr\io.h> 		/* richtige Header-Datei wird entsprechend der Einstellungen der Makedatei eingezogen*/
	#include <avr\interrupt.h>

/* Internal Defines
****************************************************************************/
	#ifndef F_CPU
		#error " dAVR_USART_H: F_CPU has not been defined -> please check your project-settings"
	#endif
	#ifndef _XTAL
		#define _XTAL		F_CPU+'UL'
		/* Taktfrequenz des Prozessors in Hz
		   >> Vorsicht mus unsigned long sein sonst kommt es zu fehlerhaften Baudraten Berechnung */
	#endif

/* Adaption for varios AVR-C
****************************************************************************/

	
	#if defined (__AVR_ATmega32__)
        //		#define UCSRA       UCSRA
        //		#define UCSRB       UCSRB
        //		#define UCSRC       UCSRC
            #define UMSEL1      URSEL 
            #define UMSEL0      UMSEL 

        //      #define UBRRH       UBRRH
        //		#define UBRRL       UBRRL

		#define USART_RX    USART_RXC_vect  

	#endif
		
	#if defined (__AVR_ATmega644__) || (__AVR_ATmega644P__)
		#define UCSRA         UCSR0A
            #define RXC         RXC0
            #define TXC         TXC0
            #define UDRE        UDRE0
            #define FE          FE0
            #define DOR         DOR0
            #define UPE         UPE0
            #define U2X         U2X0
            #define MPCM        MPCM0

		#define UCSRB         UCSR0B
		    #define TXCIE       TXCIE0
        	#define RXCIE       RXCIE0
            #define UDRIE       UDRIE0
            #define RXEN        RXEN0
            #define TXEN        TXEN0	
            #define UCSZ2       UCSZ02	
            #define RXB8        RXB80	
            #define TXB8        TXB80	

        
        #define UCSRC       UCSR0C
            #define UMSEL1      UMSEL01 
            #define UMSEL0      UMSEL00 
            #define UPM1        UPM01	
            #define UPM0        UPM00	
            #define USBS        USBS0	
            #define UCSZ1       UCSZ01	
            #define UCSZ0       UCSZ00	
            #define UCPHA       UCPHA0  
            #define UCPOL       UCPOL0	

        #define UBRRH       UBRR0H
		#define UBRRL       UBRR0L

		#define UDR         UDR0

		#define USART_RX    USART0_RX_vect   

	#endif


/* Macros
****************************************************************************/
	/**	@details	Makro calculating required content of the USART Baud Rate Register (UBRR_L) in "SINGLE SPEED MODE"
	*/
	#define dAVR_USART_UBRR_CALC(_BAUD) (((_XTAL)/((_BAUD)*16UL))-1)

	/**	@details	Makro to enable transmission via USART-interface
	*/
	#define dAVR_USART_ENABLE_TRANSMIT()		UCSRB |= ( 1 << TXEN )
	#define dAVR_USART_DISABLE_TRANSMIT()		UCSRB &= ~( 1 << TXEN )

	/**	@details	Makro to enable reveive via USART-interface
	*/
	#define dAVR_USART_ENABLE_RECEIVE()			UCSRB |= ( 1 << RXEN )
	#define dAVR_USART_DISABLE_RECEIVE()		UCSRB &= ~( 1 << RXEN )

	/**	@details	Makro to enable receive interrupt of USART-interface
	*/
	#define dAVR_USART_ENABLE_RECV_IRQ()		UCSRB |= ( 1 << RXCIE )
	#define dAVR_USART_DISABLE_RECV_IRQ()		UCSRB &= ~( 1 << RXCIE )

	/**	@details	Makro to enable transmit interrupt of USART-interface
	*/
	#define dAVR_USART_ENABLE_TRANS_IRQ()		UCSRB |= ( 1 << TXCIE )
	#define dAVR_USART_DISABLE_TRANS_IRQ()		UCSRB &= ~( 1 << TXCIE )

	/**	@details	Makro to set mode of USART-interface
		 When Synchronous mode is used (UMSEL = 1), the XCK pin will be used as either clock input
		 (Slave) or clock output (Master). The dependency between the clock edges and data sampling
		 or data change is the same. The basic principle is that data input (on RxD) is sampled at the
		 opposite XCK clock edge of the edge the data output (TxD) is changed.
	*/
	#define dAVR_USART_MODE_SYNCHRONOUS()		UCSRC |= (  1<<UMSEL0)
	#define dAVR_USART_MODE_ASYNCHRONOUS()		UCSRC &= ~( 1<<UMSEL0)

	/**	@details	Makro to set parity of USART-interface
	*/
	#define dAVR_USART_SET_PARITY_NONE()		UCSRC &= ~((1<<UPM0)|(1<<UPM1))
	#define dAVR_USART_SET_PARITY_EVEN()		dAVR_USART_SET_PARITY_NONE();	UCSRC |= (  1<<UMSEL)
	#define dAVR_USART_SET_PARITY_ODD()			dAVR_USART_SET_PARITY_NONE();	UCSRC &= ~( 1<<UMSEL)

	/**	@details	Makro to setup amount of databits for USART
			UCSZ2(in UCSRB), UCSZ1, UCSZ0	Datenbits
						0		0		0		5
						0		0		1		6
						0		1		0		7
						0		1		1		8
						1		1		1		9
	*/
	#define dAVR_USART_SET_DATABITS_TO_5()		 UCSRB &= ~(1<<UCSZ2); UCSRC &= ~(1<<UCSZ1)|(1<<UCSZ0)
	#define dAVR_USART_SET_DATABITS_TO_6()		dAVR_USART_SET_DATABITS_TO_5(); UCSRB|= (0<<UCSZ2); UCSRC |= (0<<UCSZ1)|(1<<UCSZ0)
	#define dAVR_USART_SET_DATABITS_TO_7()		dAVR_USART_SET_DATABITS_TO_5(); UCSRB|= (0<<UCSZ2); UCSRC |= (1<<UCSZ1)|(0<<UCSZ0)
	#define dAVR_USART_SET_DATABITS_TO_8()		dAVR_USART_SET_DATABITS_TO_5(); UCSRB|= (0<<UCSZ2); UCSRC |= (1<<UCSZ1)|(1<<UCSZ0)
	#define dAVR_USART_SET_DATABITS_TO_9()		dAVR_USART_SET_DATABITS_TO_5(); UCSRB|= (1<<UCSZ2); UCSRC |= (1<<UCSZ1)|(1<<UCSZ0)

	/**	@details	Makro to set number of stopp-bits used at the USART-interface
	*/

	#define dAVR_USART_SET_STOPPBITS_1()			UCSRC &= ~(1<<USBS)
	#define dAVR_USART_SET_STOPPBITS_2()			UCSRC |=  (1<<USBS)

	/**	@details	Makro to define at which edge of the XCK-signal the datasignal shall be sampled in synchronous mode.
	*/
	#define dAVR_USART_SAMPLE_AT_POS_EDGE()		UCSRC &= ~(1<<UCPOL)
	#define dAVR_USART_SAMPLE_AT_NEG_EDGE()		UCSRC |=  (1<<UCPOL)


/* Typdefinitionen
****************************************************************************/


/* Funktions-Prototypen
****************************************************************************/
	void 					avr_usart_init_9600_8_1_N_Asyn	(void);
	void 					avr_usart_putc					(uint8_t data);
	void 					avr_usart_puts 					(char *s);
	uint8_t 				avr_usart_getc_wait				(void);
	uint8_t 				avr_usart_getc					(void);
	uint8_t 				avr_usart_getc_isr				(void);

#endif /* dAVR_USART_H */


