/***************************************************************************/
/**
	@file		string_addon.c
	@brief		some additional clever string functions
	@brief		Target/Compiler: AVR_MEGA / GCC(WinAVR)
	@author		Gerhard Bruenner
	@version	V0.13


	@date		2009.12.30  - removed split_string and string_copy
						      added strtrok_single_char
	@date		2009.12.22  - changed l_trim and r_trim
							  removed access via indexes -> pure pointer
							  corrected l_trim to be l_trim and not r_trim :)
	@date		2009.10.31  - Created              - GB

	@bug		none that I am aware so far

\***************************************************************************/


#define   STRING_ADDON_C

/* Include-Dateien
****************************************************************************/
#include "string_addon.h"

/* Internal Defines
****************************************************************************/
 	
/* Function Prototypes
****************************************************************************/

/* globale int Variablen
****************************************************************************/

/* Functions
****************************************************************************/

/**
 	@brief		eleminates all leading signs which are equal to the given one
 	@param		*string - string to investigate
 	@param		sign - character which shall be eliminated
 	@return		void
 	
 	@details    This function eliminates all leading characters in a string.
				The character which shall be eliminated and the string to be 
				checked have to be addressed in the call. 
				------------ Testcode				
				sprintf(str_rcv_usart,"");
				l_trim(str_rcv_usart,_ASCII_SPACE);
				sprintf(str_rcv_usart,"a");
				l_trim(str_rcv_usart,_ASCII_SPACE);
				sprintf(str_rcv_usart,"    test x");
				l_trim(str_rcv_usart,_ASCII_SPACE);
				sprintf(str_rcv_usart,"    test mit blanks    ");
				l_trim(str_rcv_usart,_ASCII_SPACE);
 */
void l_trim(char *string, char sign) {
	char *str_ptr=string;
	if(    (_STRING_EOS!=*string)
		&& (NULL!=string) 
	){ 
		/* search for first character which is not the same like the one in char sign */
		while ( (*string!=_STRING_EOS) && (*string==sign)) string++;
		/* move content */
		while (*string!=_STRING_EOS) *(str_ptr++)=*(string++);
		*str_ptr=_STRING_EOS;	
	}	
}/* void l_trim(char *string, char sign) */


/**
 	@brief		eleminates all signs at the end of a string which are equal to the given one
 	@param		*string - string to investigate
 	@param		sign - character which shall be eliminated
 	@return		void
 	
 	@details    This function eliminates all characters at the end of a string.
				The character which shall be eliminated and the string to be 
				checked have to be addressed in the call. 
				------------ Testcode
				sprintf(str_rcv_usart,"");
				r_trim(str_rcv_usart,_ASCII_SPACE);
				sprintf(str_rcv_usart,"a");
				r_trim(str_rcv_usart,_ASCII_SPACE);
				sprintf(str_rcv_usart," test x  ");
				r_trim(str_rcv_usart,_ASCII_SPACE);
				sprintf(str_rcv_usart,"    test mit blanks    ");
				r_trim(str_rcv_usart,_ASCII_SPACE);
 */
void r_trim(char *string, char sign) {
	if(    (_STRING_EOS!=*string)
		&& (NULL!=string) 
	){ 
		/* Search for the end of the string */
		while(*string!=_STRING_EOS)	string++;
		/* search at the end for char which is not the same like the one in char sign */
		while (*(--string)==sign);
		*(++string)=_STRING_EOS;	
	}
}/* void r_trim(char *string, char sign) */


/**
 	@brief		eleminates all signs of the given type at the end or at the beginning of a string
 	@param		*string - string to investigate
 	@param		sign - character which shall be eliminated
 	@return		void
 	
 	@details    This function eliminates all characters at the end and at the beginning of a string.
				The character which shall be eliminated and the string to be 
				checked have to be addressed in the call. 
				------- Testcode
				sprintf(str_rcv_usart," op1");
				trim(str_rcv_usart,_ASCII_SPACE);
				sprintf(str_rcv_usart,"a");
				trim(str_rcv_usart,_ASCII_SPACE); 	
				sprintf(str_rcv_usart,"  help aber woher%c    ",toascii(13));
				trim(str_rcv_usart,_ASCII_SPACE); 	
				sprintf(str_rcv_usart,"");
				trim(str_rcv_usart,_ASCII_SPACE); 	
				sprintf(str_rcv_usart,"help %c",toascii(13));
				trim(str_rcv_usart,_ASCII_SPACE);  
*/
void trim(char *string, char sign) {
	char *str_ptr=string;
	if(    (_STRING_EOS!=*string)
		&& (NULL!=string) 
	){ 		/* search for first character which is not the same like the one in char sign */
		while ( (*string!=_STRING_EOS) && (*string==sign)) string++;
		/* move content */
		while (*string!=_STRING_EOS) *(str_ptr++)=*(string++);
		/* remove tailing signs */
		while (*(--str_ptr)==sign);
		*(++str_ptr)=_STRING_EOS;	
	}
}/* void trim(char *string, char sign) */

/**
 	@brief		searches for the first occurence of a single character in a string and creates a string with the content till this point.
 	@param		*c_ptr_search	- string used for search
 	@param		c_search 		- character which shall be searched
 	@return		char * - Pointer to string with content till first occurence of characeter beeing searched.
	@return				 NULL - in case character has not been found or search passed end of string.					
 	
 	@details    searches for the first occurence of a single character in a string and replaces it with '/0'.
	    	  	It returns a pointer to the beginning of the "new" string if it found the character.
	    		The function remembers the last position where a character has been found. 
				Calling the function with a c_ptr_search=NULL continuous the search at the stored location.
*/	
char *strtrok_single_char(char *c_ptr_search, char c_search){
	static char *c_static_ptr;
	char *c_ptr_to_return = NULL;

	/* check if new pointer has been forwarded */
	if(NULL!=c_ptr_search) c_static_ptr=c_ptr_search;
	c_ptr_to_return=c_static_ptr;

	/* check if pointer is "valid" */
	if(NULL	==	c_static_ptr)  			return NULL;	/* error: Null pointer */
	if(_STRING_EOS	== 	*c_static_ptr) 	return NULL;	/* error: nothing to search*/

	/* search for characted c_search in search-string */
	while( (_STRING_EOS!=*c_static_ptr) && (*c_static_ptr!=c_search))	c_static_ptr++;

	/* finalize */
	if (0!=*c_static_ptr){
		*(c_static_ptr++)=_STRING_EOS; /* Set EOS */
	}

	return c_ptr_to_return;
}/* char *strtrok_single_char(char *c_ptr_search, char c_search) */



#undef  STRING_ADDON_C

/** Messung zur V0.12 -> trim-Funktionen auf Pointer umgestellt.

	Testbedingungen:		
			Compiler-Option: -Os ? size Optimierung		
			Takt: 16 Mhz		

	Testcode:
		sprintf(str_rcv_usart,"  test op1, op2,op3, op4%c  ",toascii(13));
		trim(str_rcv_usart,_ASCII_SPACE);
		sprintf(str_rcv_usart,"  test op1, op2,op3, op4%c  ",toascii(13));
		l_trim(str_rcv_usart,_ASCII_SPACE);
		sprintf(str_rcv_usart,"  test op1, op2,op3, op4%c  ",toascii(13));
		r_trim(str_rcv_usart,_ASCII_SPACE);
				
				
	neue Version mit Pointer				
				count	time [s]
	trim 		start	8697	0
			end	9003	19,13
	l_trim		start	11756	0
			end	12040	17,75
	r_trim 		start	14793	0
			end	15032	14,94
				
	alte Version mit Index				
				count	time [s]
	trim 		start	8776	0
			end	9270	30,88
	l_trim		start	12023	0
			end	12329	19,13
	r_trim 		start	15082	0
			end	15582	31,25

*/
